local UH = UnderHood
local Module = UH:GetModule( "Bars" )
local L = Module.L
local OO = UH.OO

local LibDruidMana = LibStub( "LibDruidMana-1.0", true )

local UnitMana = UnitMana
local UnitManaMax = UnitManaMax
local UnitPowerType = UnitPowerType

local unitClass = select( 2, UnitClass( "player" ) )
local fiveSecondLength = 5

local Provider = OO:NewClass( "FiveSecondsRuleBarProvider", "BarProvider" )

function Provider:init( factory )
	super( self, factory, "AceEvent-3.0" )
end

function Provider:IsRealtime()
	return true
end

function Provider:OnAcquire( bar, zone, settings )
	super( self, bar, zone, settings )
	
	self.powerType = UnitPowerType( "player" )
	self.druidManaAttached = nil
	
	if unitClass == "DRUID" then
		self:RegisterEvent( "UNIT_DISPLAYPOWER")
		
		self.druidManaAttached = self.factory:AttachToDruidMana( self )
	end
	
	if unitClass ~= "ROGUE" and unitClass ~= "WARRIOR" then
		self.currentMana = UnitMana( "player" )
		
		self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED" )
		
		if not self.druidManaAttached then
			self:RegisterEvent( "UNIT_MANA" )
		end
	end

	self:UpdateColor()
	
	self.spellcastFinishTime = 0
	self.lastSpellcast = 0
end

function Provider:OnRelease()
	if self.druidManaAttached then
		self.factory:DetachFromDruidMana( self )
	end
	
	super( self )
end

function Provider:Show()
	self.active = true
	self.hideRequest = nil
end

function Provider:Hide()
	if self.active and not self.hideRequest then
		self.hideRequest = 1
	end
end

--
-- Called from OnUpdate of bar
--
-- arguments: currentTime = time when OnUpdate started
-- return values: value, r, g, b, a
--
function Provider:GetRealtimeValueAndColor( currentTime )
	local powerType = self.powerType
	local value

	if not self.active or powerType ~= 0 then return end
	
	if self.hideRequest and self.hideRequest > 1 then
		self.hideRequest = nil
		self.active = nil
		
		return 0
	end
	
	local timeDiff = currentTime - self.spellcastFinishTime
		
	if timeDiff > fiveSecondLength then
		self.hideRequest = 2
	else
		value = timeDiff / fiveSecondLength
	end

	return value
end

function Provider:UpdateColor()
	local r, g, b, a
	
	if self.powerType == 0 then
		r, g, b = unpack( Module.db.profile.colors.power.mana )
		a = 1
	else
		r = 0
		g = 0
		b = 0
		a = 0
	end
	
	self:SetColor( r, g, b, a )
end

function Provider:UNIT_MANA( event, unit )
	if unit ~= "player" or self.powerType ~= 0 then
		return
	end
	
	local mana = UnitMana( "player" )
	
	if mana == UnitManaMax( "player" ) then
		self:Hide()
	else
		self:Show()
		self.spellcastFinishTime = self.lastSpellcast
	end
	
	self.currentMana = mana
end

function Provider:UNIT_SPELLCAST_SUCCEEDED( event, unit )
	if unit == "player" then
		self.lastSpellcast = GetTime()
	end
end

function Provider:UpdateDruidMana( current, maximum )
	if current == maximum then
		if self.powerType == 0 then
			self:Hide()
		end
	elseif current < self.currentMana and GetTime() - self.lastSpellcast < 0.25 then
		if self.powerType == 0 then
			self:Show()
		end
		
		self.spellcastFinishTime = self.lastSpellcast
	end
	
	self.currentMana = current
end

function Provider:UNIT_DISPLAYPOWER( event, unit )
	if unit ~= "player" then return end
	
	self.powerType = UnitPowerType( "player" )

	self:UpdateColor()

	if self.powerType == 0 then
		self.currentMana = UnitMana( "player" )
		self:Show()
	else
		self:Hide()
	end
end

local Factory = OO:NewClass( "FiveSecondsRuleBarProviderFactory", "BarProviderFactory" )

function Factory:init()
	super( self, "FSR", L["Five Seconds Rule"] )
	
	if LibDruidMana then
		self.attachedProviders = {}
		
		local function notifyProviders( current, max )
			for provider, _ in pairs( self.attachedProviders ) do
				provider:UpdateDruidMana( current, max )
			end
		end
		
		LibDruidMana:AddListener( notifyProviders )
	end
end

function Factory:AttachToDruidMana( provider )
	if self.attachedProviders then
		self.attachedProviders[provider] = true
		return true
	end
end

function Factory:DetachFromDruidMana( provider )
	if self.attachedProviders then
		self.attachedProviders[provider] = nil
	end
end

function Factory:CreateProvider()
	return Provider:new( self )
end

local factory = Factory:new()

Module:RegisterProviderFactory( factory:GetName(), factory )
